package org.openfdl.text;
// $ANTLR : "OpenFDL.g" -> "FDLTreeParser.java"$

/*
 * OpenFDL Compiler
 * Copyright (C) 2005 Matt Acosta
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

import antlr.TreeParser;
import antlr.Token;
import antlr.collections.AST;
import antlr.RecognitionException;
import antlr.ANTLRException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.collections.impl.BitSet;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

import org.openfdl.core.FrameAttrNode;

/**
 * FDLTreeWalker.java
 * 
 * @author Matt Acosta
 * @version 1.00
 */


public class FDLTreeParser extends antlr.TreeParser       implements FDLTreeParserTokenTypes
 {

	// Keep track of where we started
	private FrameAttrNode rootNode = null;
	
	// Used to create/add nodes
	private FrameAttrNode posNode;
	private FrameAttrNode tempNode = new FrameAttrNode();
	
	// Slot numbers, tricky (default=0x10000)
	private int tempNum, lastNum = 65535;
	
	private String tempStr;
	
	private boolean errorFound = false;
	
	public FrameAttrNode getRootNode()
	{
		return rootNode;
	}
public FDLTreeParser() {
	tokenNames = _tokenNames;
}

	public final void parse(AST _t) throws RecognitionException {
		
		AST parse_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			{
			_loop718:
			do {
				if (_t==null) _t=ASTNULL;
				if (((_t.getType() >= LITERAL_var && _t.getType() <= LITERAL_appfunc))) {
					fdlLine(_t);
					_t = _retTree;
					
							if (!errorFound)
							{
						  		// Set slot numbers (we increment to default if missing)
						  		if (tempNum == 0)
						 	 		lastNum++;
						 	 	else
						 	 		lastNum = tempNum;
						 	 	
						 	 	tempNode.setSlotNum(lastNum);
						  		
						  		// Reset fields
						  		tempNum = 0;
						  		tempStr = null;
						  		
						  		// Add to the nodes..
						  		if (rootNode == null)
						  			posNode = rootNode = tempNode;
						  		else
						  			posNode = posNode.appendAttrNode(tempNode);
							}
							
							tempNode = new FrameAttrNode();
							
							errorFound = false;
					
				}
				else {
					break _loop718;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void fdlLine(AST _t) throws RecognitionException {
		
		AST fdlLine_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case LITERAL_var:
			case LITERAL_appvar:
			{
				varDecl(_t);
				_t = _retTree;
				break;
			}
			case LITERAL_func:
			{
				funcDecl(_t);
				_t = _retTree;
				break;
			}
			case LITERAL_appfunc:
			{
				appFuncDecl(_t);
				_t = _retTree;
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void varDecl(AST _t) throws RecognitionException {
		
		AST varDecl_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST attrType = null;
		AST slotNum = null;
		AST attrName = null;
		
		try {      // for error handling
			attrType = _t==ASTNULL ? null : (AST)_t;
			varType(_t);
			_t = _retTree;
			
					tempNode.setAttrType(attrType.getType());
				
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case INTLIT:
			{
				slotNum = (AST)_t;
				match(_t,INTLIT);
				_t = _t.getNextSibling();
				
					tempNum = Integer.parseInt(slotNum.getText());
				
				break;
			}
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			attrName = (AST)_t;
			match(_t,IDENT);
			_t = _t.getNextSibling();
			
				tempNode.setAttrName(attrName.getText());
			
			AST tmp38_AST_in = (AST)_t;
			match(_t,COLON);
			_t = _t.getNextSibling();
			tempStr = "";
			typeSpec(_t);
			_t = _retTree;
			
				tempNode.setVarType(tempStr.trim());
			
			AST tmp39_AST_in = (AST)_t;
			match(_t,SEMI);
			_t = _t.getNextSibling();
		}
		catch (RecognitionException e) {
			
				errorFound = true;
				reportError(e.getMessage());
			
		}
		_retTree = _t;
	}
	
	public final void funcDecl(AST _t) throws RecognitionException {
		
		AST funcDecl_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST attrType = null;
		AST slotNum = null;
		AST attrName = null;
		
		try {      // for error handling
			attrType = _t==ASTNULL ? null : (AST)_t;
			funcType(_t);
			_t = _retTree;
			
					tempNode.setAttrType(attrType.getType());
			
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case INTLIT:
			{
				slotNum = (AST)_t;
				match(_t,INTLIT);
				_t = _t.getNextSibling();
				
					tempNum = Integer.parseInt(slotNum.getText());
				
				break;
			}
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			attrName = (AST)_t;
			match(_t,IDENT);
			_t = _t.getNextSibling();
			
				tempNode.setAttrName(attrName.getText());
			
			funcParameters(_t);
			_t = _retTree;
			AST tmp40_AST_in = (AST)_t;
			match(_t,COLON);
			_t = _t.getNextSibling();
			tempStr = "";
			typeSpec(_t);
			_t = _retTree;
			
				tempNode.setVarType(tempStr.trim());
			
			AST tmp41_AST_in = (AST)_t;
			match(_t,SEMI);
			_t = _t.getNextSibling();
		}
		catch (RecognitionException e) {
			
				errorFound = true;
				reportError(e.getMessage());
			
		}
		_retTree = _t;
	}
	
	public final void appFuncDecl(AST _t) throws RecognitionException {
		
		AST appFuncDecl_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST attrType = null;
		AST slotNum = null;
		AST attrName = null;
		
		try {      // for error handling
			attrType = _t==ASTNULL ? null : (AST)_t;
			appFuncType(_t);
			_t = _retTree;
			
					tempNode.setAttrType(attrType.getType());
			
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case INTLIT:
			{
				slotNum = (AST)_t;
				match(_t,INTLIT);
				_t = _t.getNextSibling();
				
					tempNum = Integer.parseInt(slotNum.getText());
				
				break;
			}
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			attrName = (AST)_t;
			match(_t,IDENT);
			_t = _t.getNextSibling();
			
				tempNode.setAttrName(attrName.getText());
			
			appFuncParameters(_t);
			_t = _retTree;
			AST tmp42_AST_in = (AST)_t;
			match(_t,COLON);
			_t = _t.getNextSibling();
			tempStr = "";
			typeSpec(_t);
			_t = _retTree;
			
				tempNode.setVarType(tempStr.trim());
			
			AST tmp43_AST_in = (AST)_t;
			match(_t,SEMI);
			_t = _t.getNextSibling();
		}
		catch (RecognitionException e) {
			
				errorFound = true;
				reportError(e.getMessage());
			
		}
		_retTree = _t;
	}
	
	public final void varType(AST _t) throws RecognitionException {
		
		AST varType_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case LITERAL_var:
			{
				AST tmp44_AST_in = (AST)_t;
				match(_t,LITERAL_var);
				_t = _t.getNextSibling();
				break;
			}
			case LITERAL_appvar:
			{
				AST tmp45_AST_in = (AST)_t;
				match(_t,LITERAL_appvar);
				_t = _t.getNextSibling();
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void typeSpec(AST _t) throws RecognitionException {
		
		AST typeSpec_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST attr = null;
		AST ptr = null;
		
		try {      // for error handling
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case LITERAL_const:
			{
				attr = _t==ASTNULL ? null : (AST)_t;
				typeAttr(_t);
				_t = _retTree;
				
						tempStr += " " + attr.getText();
					
				break;
			}
			case IDENT:
			case LITERAL_char:
			case LITERAL_int:
			case LITERAL_long:
			case LITERAL_short:
			case LITERAL_float:
			case LITERAL_double:
			case LITERAL_struct:
			case LITERAL_union:
			case LITERAL_unsigned:
			case LITERAL_signed:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			typeName(_t);
			_t = _retTree;
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case TIMES:
			{
				ptr = (AST)_t;
				match(_t,TIMES);
				_t = _t.getNextSibling();
				
					tempStr += " " + ptr.getText();
				
				break;
			}
			case SEMI:
			case COMMA:
			case RPAREN:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
		}
		catch (RecognitionException e) {
			
				errorFound = true;
				reportError("invalid type");
			
		}
		_retTree = _t;
	}
	
	public final void funcType(AST _t) throws RecognitionException {
		
		AST funcType_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			AST tmp46_AST_in = (AST)_t;
			match(_t,LITERAL_func);
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void funcParameters(AST _t) throws RecognitionException {
		
		AST funcParameters_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST pFrameStr = null;
		AST comma = null;
		
		try {      // for error handling
			AST tmp47_AST_in = (AST)_t;
			match(_t,LPAREN);
			_t = _t.getNextSibling();
			pFrameStr = _t==ASTNULL ? null : (AST)_t;
			funcReqParam(_t);
			_t = _retTree;
			
				tempStr = pFrameStr.getText();
			
			{
			_loop731:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==COMMA)) {
					comma = (AST)_t;
					match(_t,COMMA);
					_t = _t.getNextSibling();
					
						tempStr += comma.getText();
					
					typeSpec(_t);
					_t = _retTree;
				}
				else {
					break _loop731;
				}
				
			} while (true);
			}
			AST tmp48_AST_in = (AST)_t;
			match(_t,RPAREN);
			_t = _t.getNextSibling();
			tempNode.setParams(tempStr);
		}
		catch (RecognitionException e) {
			
				errorFound = true;
				reportError(e.getMessage());
			
		}
		_retTree = _t;
	}
	
	public final void appFuncType(AST _t) throws RecognitionException {
		
		AST appFuncType_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			AST tmp49_AST_in = (AST)_t;
			match(_t,LITERAL_appfunc);
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void appFuncParameters(AST _t) throws RecognitionException {
		
		AST appFuncParameters_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST appid = null;
		AST comma = null;
		
		try {      // for error handling
			AST tmp50_AST_in = (AST)_t;
			match(_t,LPAREN);
			_t = _t.getNextSibling();
			appid = _t==ASTNULL ? null : (AST)_t;
			appFuncReqParam(_t);
			_t = _retTree;
			
				tempStr = appid.getText();
			
			{
			_loop734:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==COMMA)) {
					comma = (AST)_t;
					match(_t,COMMA);
					_t = _t.getNextSibling();
					
						tempStr += comma.getText();
					
					typeSpec(_t);
					_t = _retTree;
				}
				else {
					break _loop734;
				}
				
			} while (true);
			}
			AST tmp51_AST_in = (AST)_t;
			match(_t,RPAREN);
			_t = _t.getNextSibling();
			tempNode.setParams(tempStr);
		}
		catch (RecognitionException e) {
			
				errorFound = true;
				reportError(e.getMessage());
			
		}
		_retTree = _t;
	}
	
	public final void funcReqParam(AST _t) throws RecognitionException {
		
		AST funcReqParam_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			AST tmp52_AST_in = (AST)_t;
			match(_t,LITERAL_pFrame);
			_t = _t.getNextSibling();
		}
		catch (RecognitionException e) {
			
				errorFound = true;
				reportError(e.getMessage());
			
		}
		_retTree = _t;
	}
	
	public final void appFuncReqParam(AST _t) throws RecognitionException {
		
		AST appFuncReqParam_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			AST tmp53_AST_in = (AST)_t;
			match(_t,LITERAL_AppID);
			_t = _t.getNextSibling();
		}
		catch (RecognitionException e) {
			
				errorFound = true;
				reportError(e.getMessage());
			
		}
		_retTree = _t;
	}
	
	public final void typeAttr(AST _t) throws RecognitionException {
		
		AST typeAttr_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			AST tmp54_AST_in = (AST)_t;
			match(_t,LITERAL_const);
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void typeName(AST _t) throws RecognitionException {
		
		AST typeName_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST dType = null;
		AST name = null;
		AST mod = null;
		AST nVal = null;
		AST dVal = null;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			case LITERAL_struct:
			case LITERAL_union:
			{
				{
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case LITERAL_struct:
				case LITERAL_union:
				{
					dType = _t==ASTNULL ? null : (AST)_t;
					dataType(_t);
					_t = _retTree;
					tempStr += " " + dType.getText();
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				name = (AST)_t;
				match(_t,IDENT);
				_t = _t.getNextSibling();
				tempStr += " " + name.getText();
				}
				break;
			}
			case LITERAL_char:
			case LITERAL_int:
			case LITERAL_long:
			case LITERAL_short:
			case LITERAL_unsigned:
			case LITERAL_signed:
			{
				{
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case LITERAL_unsigned:
				case LITERAL_signed:
				{
					mod = _t==ASTNULL ? null : (AST)_t;
					typeMod(_t);
					_t = _retTree;
					tempStr += " " + mod.getText();
					break;
				}
				case LITERAL_char:
				case LITERAL_int:
				case LITERAL_long:
				case LITERAL_short:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				nVal = _t==ASTNULL ? null : (AST)_t;
				numericType(_t);
				_t = _retTree;
				tempStr += " " + nVal.getText();
				}
				break;
			}
			case LITERAL_float:
			case LITERAL_double:
			{
				dVal = _t==ASTNULL ? null : (AST)_t;
				decimalType(_t);
				_t = _retTree;
				tempStr += " " + dVal.getText();
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void dataType(AST _t) throws RecognitionException {
		
		AST dataType_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case LITERAL_struct:
			{
				AST tmp55_AST_in = (AST)_t;
				match(_t,LITERAL_struct);
				_t = _t.getNextSibling();
				break;
			}
			case LITERAL_union:
			{
				AST tmp56_AST_in = (AST)_t;
				match(_t,LITERAL_union);
				_t = _t.getNextSibling();
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void typeMod(AST _t) throws RecognitionException {
		
		AST typeMod_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case LITERAL_unsigned:
			{
				AST tmp57_AST_in = (AST)_t;
				match(_t,LITERAL_unsigned);
				_t = _t.getNextSibling();
				break;
			}
			case LITERAL_signed:
			{
				AST tmp58_AST_in = (AST)_t;
				match(_t,LITERAL_signed);
				_t = _t.getNextSibling();
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void numericType(AST _t) throws RecognitionException {
		
		AST numericType_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case LITERAL_char:
			{
				AST tmp59_AST_in = (AST)_t;
				match(_t,LITERAL_char);
				_t = _t.getNextSibling();
				break;
			}
			case LITERAL_int:
			{
				AST tmp60_AST_in = (AST)_t;
				match(_t,LITERAL_int);
				_t = _t.getNextSibling();
				break;
			}
			case LITERAL_long:
			{
				AST tmp61_AST_in = (AST)_t;
				match(_t,LITERAL_long);
				_t = _t.getNextSibling();
				break;
			}
			case LITERAL_short:
			{
				AST tmp62_AST_in = (AST)_t;
				match(_t,LITERAL_short);
				_t = _t.getNextSibling();
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final void decimalType(AST _t) throws RecognitionException {
		
		AST decimalType_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case LITERAL_float:
			{
				AST tmp63_AST_in = (AST)_t;
				match(_t,LITERAL_float);
				_t = _t.getNextSibling();
				break;
			}
			case LITERAL_double:
			{
				AST tmp64_AST_in = (AST)_t;
				match(_t,LITERAL_double);
				_t = _t.getNextSibling();
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"INTLIT",
		"IDENT",
		":",
		";",
		"\"var\"",
		"\"appvar\"",
		"\"func\"",
		"\"appfunc\"",
		"(",
		",",
		")",
		"\"pFrame\"",
		"\"AppID\"",
		"*",
		"\"char\"",
		"\"int\"",
		"\"long\"",
		"\"short\"",
		"\"float\"",
		"\"double\"",
		"\"struct\"",
		"\"union\"",
		"\"const\"",
		"\"unsigned\"",
		"\"signed\"",
		"WS",
		"COMMENT",
		"LBRAKET",
		"RBRAKET"
	};
	
	}
	
